@@ -4,3 +4,5 @@ CACHE = False
 DEBUG = False
 # Smart errors
 SMART_ERRORS = False
+# Transactions
+TX_NAME = "_tx"
