@@ -16,8 +16,16 @@ class NodesTestCase(unittest.TestCase):
     def setUp(self):
         self.url = "http://localhost:7474/db/data/"
         self.gdb = client.GraphDatabase(self.url)
+        # self.secured_url = "http://<instance>.hosted.neo4j.org:7000/db/data/"
+        # self.username = "username"
+        # self.password = "password"
+        # self.gdb = client.GraphDatabase(self.secured_url, username=self.username,
+        #                                 password=self.password)
 
     def tearDown(self):
+        import options as clientCacheDebug
+        clientCacheDebug.DEBUG = False
+        clientCacheDebug.CACHE = False
         if self.gdb:
             self.gdb.flush()
 
@@ -464,8 +472,6 @@ class TraversalsTestCase(IndexesTestCase):
         """
         Tests the use of paginated traversals.
         """
-        import options as clientDebug
-        clientDebug.DEBUG = False
         nodes = [self.gdb.nodes.create() for i in xrange(10)]
         # Chain them into a linked list
         last = None
